<?php

use Illuminate\Support\HtmlString;
use Illuminate\Support\Collection;
use Illuminate\Support\Str;


if ( !function_exists('str_singularize_fr')) {
    /**
     * Converti une chaine de caractères pluriel en singulier (Francais)
     * @param  string $string La chaine initiale
     * @return string         La chaine resultante
     */
    function str_singularize_fr(string $string)
    {
        $replacements = [
            'mes'        => '(mon|ma)',
            'les'        => '(le|la)',
            'arsenaux'   => 'arsenal',
            'maux'       => 'mal',
            'idéaux'     => 'idéal',
            'chevaux'    => 'cheval',
            'travaux'    => 'travail',
            'bus'        => 'bus',
            'os'         => 'os',
            'brebis'     => 'brebis',
            'bus'        => 'bus',
            'cas'        => 'cas',
            'fils'       => 'fils',
            'gaz'        => 'gaz',
            'héros'      => 'héros',
            'houx'       => 'houx',
            'index'      => 'index',
            'lynx'       => 'lynx',
            'matelas'    => 'matelas',
            'nez'        => 'nez',
            'puits'      => 'puits',
            'prix'       => 'prix',
            'quartz'     => 'quartz',
            'quiz'       => 'quiz',
            'rabais'     => 'rabais',
            'rhinocéros' => 'rhinocéros',
            'sas'        => 'sas',
            'souris'     => 'souris',
            'taux'       => 'taux',
            'bijoux'     => 'bijou',
            'cailloux'   => 'caillou',
            'choux'      => 'chou',
            'genoux'     => 'genou',
            'hiboux'     => 'hibou',
            'joujoux'    => 'joujou',
            'poux'       => 'pou',
            'ripoux'     => 'ripou',
            'cadeaux'    => 'cadeau',
            'vœux'       => 'vœu'
        ];

        // Remplacement des termes
        $string = str_replace(array_keys($replacements), array_values($replacements), $string);

        // Suppression des « s » en fin de phrase
        $string = preg_replace('/([sx])[^a-z]?$/i', '', $string);

        return $string;
    }
}

if ( !function_exists('str_indent')) {
    /**
     * Indente une chaine de caractères
     * @param  string $text   La chaine à indenter
     * @param  int $nb_indent Le nombre d'indentation à effectuer
     * @return string         La chaine de caractères resultante
     */
    function str_indent(string $text, $nb_indent)
    {
        $indent = '    ';
        $render = '';

        $real_indent = '';

        for ($i = 1; $i <= $nb_indent; $i++) {
            $real_indent .= $indent;
        }

        $lines = explode(PHP_EOL, $text);

        foreach ($lines as $line) {

            if (!empty($line)) {
                $render .= $real_indent . $line . PHP_EOL;
            }
        }

        return $render;
    }
}

if (! function_exists('str_args_to_array')) {
    /**
     * Converti une chaine d'arguments en tableau d'arguments en respectant le typage
     *
     * @param string $value La chaine à convertir
     * @return array        Le tableau d'arguments
     */
    function str_args_to_array(string $value) : array
    {
        $args = explode(',', $value);
        $args = array_map(function($el) {
            $el = trim($el);

            if (Str::contains($el, ['"', "'"])) {
                return trim($el, '\'"');
            } else if (preg_match('/^\d{1,}$/', $el)) {
                return intval($el);
            } else if (preg_match('/^\d{1,}.\d{1,}$/', $el)) {
                return floatval($el);
            } else if (preg_match('/^true|false$/', $el)) {
                return boolval($el);
            } else {
                return null;
            }
        }, $args);

        return $args;
    }
}